import { HttpClient } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { environment } from "environments/environment";
import { KeycloakEventType, KeycloakService } from "keycloak-angular";
import { BehaviorSubject, from, Subject } from "rxjs";
import { Observable, of } from "rxjs";
import { catchError, map, flatMap } from "rxjs/operators";
import { SubatoPrincipal } from "../models/principal/subato-principal";

@Injectable({
    providedIn: 'root'
})
export class AuthService {
    principal: SubatoPrincipal;
    private _principal: Subject<SubatoPrincipal>;
    principal$: Observable<SubatoPrincipal>;

    constructor(private http: HttpClient, private keycloakService: KeycloakService) {
        this._principal = new BehaviorSubject<SubatoPrincipal>(null);
        this.principal$ = this._principal.asObservable();
        this.principal$.subscribe(principal => this.principal = principal);

        this.keycloakService.keycloakEvents$.subscribe(evt => {
            if (evt.type == KeycloakEventType.OnAuthLogout) {
                this._principal.next(null);
                this.notifyLogout()
                    .pipe(
                        map(() => {
                            this.keycloakService.login({
                                redirectUri: window.location.href
                            });
                        })
                    )
            }
        })
    }

    loadPrincipal(): Observable<SubatoPrincipal> {
        return from(this.keycloakService.getToken())
            .pipe(
                flatMap(token => {
                    if (!token) {
                        throw "Not signed in";
                    }
                    return this.getPrincipal();
                }),
                map((principal) => {
                    this._principal.next(principal);
                    return principal;
                }),
                catchError(() => of(null))
            );
    }

    private getPrincipal(): Observable<SubatoPrincipal> {
        return this.http.get<SubatoPrincipal>(`${environment.backendUrl}/auth/principal`)
            .pipe(
                map(principal => new SubatoPrincipal(principal))
            );
    }

    isAuthenticated(): boolean {
        return this.principal != null;
    }

    login(): Observable<void> {
        return from(this.keycloakService.login({
            redirectUri: window.location.href
        }));
    }

    logout(): Observable<void> {
        return this.notifyLogout()
            .pipe(
                flatMap(() => from(this.keycloakService.logout(window.location.origin))),
                map(() => {
                    this._principal.next(null);
                }),
            );
    }

    notifyLogout(): Observable<void> {
        return this.http.post<void>(`${environment.backendUrl}/auth/logout`, {});
    }
}