import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { User } from 'app/core/models/user';
import { AppStateService } from 'app/core/services/app.state.service';
import { AuthService } from 'app/core/services/auth.service';
import { Course } from 'app/features/courses/models/course';
import { CourseUnit } from 'app/features/courses/models/course-unit';
import { CourseService } from 'app/features/courses/services/course.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin, of } from 'rxjs';
import { takeUntil, flatMap, filter, catchError } from 'rxjs/operators';


@Component({
  selector: 'ngx-course-results',
  templateUrl: './course-results.component.html',
  styleUrls: ['./course-results.component.scss']
})
export class CourseResultsComponent extends BaseComponent implements OnInit {
  course: Course;
  units: CourseUnit[] = [];
  loading: boolean;
  principal: SubatoPrincipal;
  onlyOwnUnits: boolean = true;
  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Kurs konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(
    private courseService: CourseService,
    private router: Router,
    private appStateService: AppStateService,
    private authService: AuthService,
    private notificationService: NotificationService,
    private route: ActivatedRoute) {
    super();
  }

  buildBreadcrumbs(course: Course) {
    this.breadcrumbs = [
      {
        id: 'courseResults',
        params: {
          course_id: course.id
        },
      },
    ];
  }

  get filteredUnits(): CourseUnit[] {
    return this.units
      .filter(u => u.isLab)
      .filter(unit => !this.onlyOwnUnits || unit.lecturers.some(l => l.loginName == this.principal.loginName));
  }

  ngOnInit(): void {
    this.loading = true;
    
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let courseId = Number(params.get("course_id"));

          return forkJoin(
            this.courseService.getById(courseId),
            this.courseService.getUnits(courseId)
          );
        }),
      )
      .subscribe(([course, units]) => {
        this.course = course;
        this.onlyOwnUnits = this.principal != null && this.principal.isLecturerOrAdmin(course.id);

        this.appStateService.setCourse(course);
        this.buildBreadcrumbs(course);
        this.units = units;
        this.loading = false;
      }, (error) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

  getUnitLink(unit: CourseUnit) {
      return ['/', 'course', unit.course.id, 'results', unit.id];
  }
}
