import { Component, OnInit } from '@angular/core';
import { User } from 'app/core/models/user';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { ParticipantAttendance } from 'app/features/attendances/dto/participant-attendance.dto';
import { AttendanceService } from 'app/features/attendances/services/attendance.service';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin } from 'rxjs';
import { takeUntil, flatMap, map } from 'rxjs/operators';
import * as moment from 'moment';
import { BackendError } from 'app/shared/models/backend-error';
import { ExerciseProgress } from 'app/features/exercises/models/exercise-progress';
import { ExerciseService } from 'app/features/exercises/services/exercise.service';
import { CourseUnitEvent } from 'app/features/courses/models/course-unit-event';
import { Course } from 'app/features/courses/models/course';
import { CourseService } from 'app/features/courses/services/course.service';
import { UserService } from 'app/core/services/user.service';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { ExerciseProgressResult } from 'app/features/exercises/models/exercise-progress-result';

@Component({
  selector: 'ngx-participant-progress',
  templateUrl: './participant-progress.component.html',
  styleUrls: ['./participant-progress.component.scss']
})
export class ParticipantProgressComponent extends BaseComponent implements OnInit {
  loading: boolean = false;
  user: User;
  principal: SubatoPrincipal;
  userId: number;
  course: Course;
  participantAttendances: ParticipantAttendance[];
  events: CourseUnitEvent[];
  exerciseProgressResult: ExerciseProgressResult;
  hideInvisible;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Kurs konnten wir leider nicht finden.'
    }
  };

  constructor(private courseService: CourseService,
    private exerciseService: ExerciseService,
    private appStateService: AppStateService,
    private userService: UserService,
    private attendanceService: AttendanceService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private route: ActivatedRoute,
    private router: Router) {
    super();
  }

  get canSeeFilter() {
    return this.principal != null && this.principal.hasAnyGroup(this.course.id);
  }

  ngOnInit(): void {
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, user]) => {
          this.loading = true;
          this.principal = user;
          let courseId = Number(params.get("course_id"));
          let userId = params.has("user_id") ? Number(params.get("user_id")) : user.id;
          this.userId = userId;
          return this.courseService.getById(courseId);   
        }),
        flatMap((course) => {
          this.course = course;
          this.appStateService.setCourse(course);
          this.hideInvisible = this.canSeeFilter;
          return this.initUser(this.userId);
        })
      )
      .subscribe(() => {
        this.loading = false;
      }, (error: any) => {

        if(BackendError.isBackendError(error)) {
            this.error = error;
            this.loading = false;
        } else {
          throw error;
        }
      });
  }

  initUser(userId: number) {
    return forkJoin(
      this.attendanceService.getAttendancesForUser(this.course.id, userId), 
      this.exerciseService.getProgressForUser(this.course.id, userId),
      this.userService.getById(userId)
    )
    .pipe(
      map(([attendances, exerciseProgressResult, user]) => {
        this.user = user;
        this.participantAttendances = attendances;
        this.exerciseProgressResult = exerciseProgressResult;
      })
    );
  }
}
