import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { forkJoin, Observable, of } from "rxjs";
import { catchError, flatMap, map, tap } from "rxjs/operators";
import { BackendError } from "app/shared/models/backend-error";
import { Page } from "app/shared/models/page";
import { DataQueryParams } from "app/shared/models/data-query-params";
import { FileTree } from 'app/shared/models/file-tree';
import { DownloadService } from "app/shared/services/download.service";
import { Client } from "../models/client";
import { Solution } from "../models/solution";
import { EvalResult } from "../models/eval-result";

export interface SolutionFilter {
    exerciseId?: number;
    taskId?: number;
    userId?: number;
    submissionId?: number;
    valid?: boolean
}

export interface SubmitFileSolution {
    exerciseId: number;
    taskId: number;
    comment: String;
    lateSubmit: boolean;
    submittedWith: Client;
}

export interface SubmitInlineSolution {
    exerciseId: number;
    taskId: number;
    comment: String;
    lateSubmit: boolean;
    content: string;
}

@Injectable({
    providedIn: 'root'
})
export class SolutionService extends BaseService {
    constructor(private http: HttpClient, private downloadService: DownloadService) {
        super(); 
    }

    getById(id: number): Observable<Solution> {
        return this.http.get<Solution>(`${environment.backendUrl}/solution/${id}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(solution => new Solution(solution))
        );
    }

    getSolutions(filter: SolutionFilter, projection?: DataQueryParams): Observable<Page<Solution>> {
        var params = DataQueryParams.toHttpParams(projection);

        if(filter.userId != null) {
            params = params.set('userId', filter.userId);
        }

        if(filter.exerciseId != null) {
            params = params.set('exerciseId', filter.exerciseId);
        }

        if(filter.taskId != null) {
            params = params.set('taskId', filter.taskId);
        }

        if(filter.submissionId != null) {
            params = params.set('submissionId', filter.submissionId);
        }

        if(filter.valid != null) {
            params = params.set('valid', filter.valid);
        }

        return this.http.get<Page<Solution>>(`${environment.backendUrl}/solution`, {
            params: params
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(page => new Page<Solution>(page, item => new Solution(item)))
        );
    }

    submitFileSolution(dto: SubmitFileSolution, file: File | null): Observable<Solution> {
        var formData = new FormData();
        if(file) {
            formData.append("file", file);
        }
        
        formData.append("solution", new Blob([JSON.stringify(dto)], { type: 'application/json' }));

        return this.http.post<Solution>(`${environment.backendUrl}/solution/submit`, formData).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(response => new Solution(response))
        );
    }

    submitInlineSolution(dto: SubmitInlineSolution): Observable<Solution> {
        return this.http.post<Solution>(`${environment.backendUrl}/solution/submit`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(response => new Solution(response))
        );
    }

    runAcceptanceTests(solutionId: number): Observable<EvalResult> {
        return this.http.post<Solution>(`${environment.backendUrl}/solution/${solutionId}/acceptancetest`, {}).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(response => new EvalResult(response))
        );
    }

    getFileUrl(solutionId: number, uuid: string, download?: boolean): string {
        var url = `${environment.backendUrl}/solution/${solutionId}/file/${uuid}`;
        if(download) {
            url += '?download=true';
        }

        return url;
    }

    getFileAsUrl(solutionId: number, item: FileTree): Observable<[url: string, fileName: string]> {
        return this.downloadService.downloadFileAsUrl(this.getFileUrl(solutionId, item.uuid));
    }

    getFile(solutionId: number, item: FileTree): Observable<string> {
        return this.http.get(this.getFileUrl(solutionId, item.uuid), {
            responseType: 'text',
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map((data: any) => data)
        );
    }

    downloadFile(solutionId: number, item: FileTree): Observable<void> {
        return this.downloadService.downloadFile(this.getFileUrl(solutionId, item.uuid));
    }

    downloadSolution(solutionId: number): Observable<void> {
        return this.downloadService.downloadFile(`${environment.backendUrl}/solution/${solutionId}/download`);
    }
}