import { Component, Input, OnInit } from '@angular/core';
import { AbstractControl, UntypedFormBuilder, UntypedFormGroup, ValidationErrors, ValidatorFn, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { FormComponent } from 'app/shared/components/base/form.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { environment } from 'environments/environment';
import { takeUntil } from 'rxjs/operators';
import { CreateTaskPoolDto } from '../../dto/create-task-pool.dto';
import { TaskPool } from '../../models/task-pool';
import { TaskPoolService } from '../../services/task-pool.service';

export const repositoryValidator: ValidatorFn = (control: AbstractControl): ValidationErrors | null => {
  const useRepository = control.get('useRepository').value;
  const repositoryUrl = control.get('repositoryUrl').value;

  return useRepository && !repositoryUrl ? { repositoryUrlMissing: true } : null;
};

@Component({
  selector: 'ngx-task-pool-form',
  templateUrl: './task-pool-form.component.html',
  styleUrls: ['./task-pool-form.component.scss']
})
export class TaskPoolFormComponent extends FormComponent implements OnInit {
  @Input() pool: TaskPool;
  
  saving: boolean = false;
  poolForm: UntypedFormGroup;

  tasksManualUrl = environment.tasksManualUrl;
  
  constructor(
    private poolService: TaskPoolService,
    private router: Router,
    private route: ActivatedRoute,
    private notificationService: NotificationService,
    private fb: UntypedFormBuilder) {
    super();
  }

  createForm(pool?: TaskPool) {
    this.poolForm = this.fb.group({
      name: [pool?.name ?? '', Validators.required],
      description: [pool?.description ?? ''],
      useRepository: [pool?.linkedToRepository ?? false],
      repositoryUrl: [pool?.repositoryUrl ? pool.repositoryUrl : ''],
    }, { validators: repositoryValidator });
  }

  ngOnInit(): void {
    this.createForm(this.pool);
  }

  get useRepository(): boolean {
    return this.poolForm.get('useRepository').value;
  }

  cancel() {
    this.router.navigate(['task_pools']);
  }

  save() {
    this.saving = true;

    let dto: CreateTaskPoolDto = {
      name: this.poolForm.get('name').value,
      description: this.poolForm.get('description').value,
      repositoryUrl: this.useRepository 
        ? this.poolForm.get('repositoryUrl').value 
        : null,
    };

    if (this.pool != null) {
      this.poolService.update(this.pool.id, dto)
        .pipe(takeUntil(this.unsubscribe))
        .subscribe(pool => {
          this.saving = false;
          this.notificationService.showSuccess("Pool erfolgreich aktualisiert");
          this.router.navigate(['task_pools', pool.id]);
        }, (error: any) => {
          this.saving = false;
          if (BackendError.isBackendError(error)) {
            if (error.isValidation()) {
              this.errors = this.getValidationErrors(error);
            } 
  
            this.notificationService.showError(
              error.format(
                "Fehler bei Aktualisierung des Pools",
              )
            );
          } else {
            throw error;
          }
        });
    } else {
      this.poolService.create(dto)
        .pipe(takeUntil(this.unsubscribe))
        .subscribe(result => {
          this.saving = false;
          if(!result.syncFailed && (result.taskImportResult == null || result.taskImportResult.problems == 0)) {
            this.notificationService.showSuccess("Pool erfolgreich erstellt");
          } else {
            this.notificationService.showWarning("Pool erstellt aber bei der Synchronisation sind Probleme aufgetreten.");
          }

          this.router.navigate(['task_pools', result.pool.id]);
        }, (error: any) => {
          this.saving = false;
          if (BackendError.isBackendError(error)) {
            this.notificationService.showError(
              error.format(
                "Fehler bei Erstellung des Pools" 
              )
            );
          } else {
            throw error;
          }
        });
    }
  }
}
