import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Task } from 'app/features/tasks/models/task';
import { User } from 'app/core/models/user';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { TaskService } from 'app/features/tasks/services/task.service';
import { combineLatest, forkJoin, of } from 'rxjs';
import { takeUntil, flatMap, filter } from 'rxjs/operators';
import { TaskFeedback } from '../../models/task-feedback';
import { FileRef } from '../../models/file-ref';
import { BackendError } from 'app/shared/models/backend-error';
import { NbDialogService } from '@nebular/theme';
import { DialogConfirmationComponent } from 'app/shared/dialogs/dialog-confirmation/dialog-confirmation.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';

@Component({
  selector: 'ngx-task',
  templateUrl: './task.component.html',
  styleUrls: ['./task.component.scss']
})
export class TaskComponent extends BaseComponent implements OnInit {
  loading: boolean;
  task: Task;
  taskContent: string;
  principal: SubatoPrincipal;
  archivingTask = false;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Aufgabe konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];
  
  constructor(
    private authService: AuthService,
    private taskService: TaskService,
    private dialogService: NbDialogService,
    private router: Router,
    private route: ActivatedRoute,
    private notificationService: NotificationService,
  ) {
    super();
  }

  buildBreadcrumbs(task: Task) {
    this.breadcrumbs = [
      {
        id: 'pools',
      },
      {
        id: 'pool',
        params: {
          pool_id: task.pool.id
        },
        ctx: task.pool
      },
      {
        id: 'task',
        params: {
          task_id: task.id
        },
        ctx: task
      },
    ];
  }

  ngOnInit(): void {
    this.loading = true;
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let taskId = Number(params.get("task_id"));
          return forkJoin(this.taskService.getById(taskId), this.taskService.getContent(taskId)); 
        }),
        ).subscribe(([task, content]) => {
          this.task = task;
          this.taskContent = content;
          this.buildBreadcrumbs(task);
          this.loading = false;
        }, error => {
          if (BackendError.isBackendError(error)) {
            this.error = error;
            this.loading = false;
          } else {
            throw error;
          }
      });
  }

  downloadSpec() {
    this.taskService.downloadSpec(this.task.id)
    .pipe(takeUntil(this.unsubscribe))
    .subscribe(() => {}, error => {
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Herunterladen der Aufgabe",
          )
        );
      } else {
        throw error;
      }
    });
  }

  archiveTask() {
    this.dialogService.open(DialogConfirmationComponent, {
      context: {
        title: `Archivierung von Aufgabe: ${this.task.name}`,
        text: `Bist du wirklich sicher, dass du die Aufgabe '${this.task.name}' archivieren willst? Archivierte Aufgaben sind nicht mehr verfügbar und assoziierte Lösungen können nicht mehr direkt eingesehen werden. Die Aufgabe und die Verbindungen zu Übungsblättern und Lösungen bleiben jedoch erhalten und können von Admins wiederhergestellt werden.`,
        actionText: 'Archivieren'
      }
    }).onClose.pipe(
      takeUntil(this.unsubscribe),
      filter(result => result && result.confirmed),
      flatMap(() => {
        this.archivingTask = true;
        return this.taskService.archive(this.task.id);
      })
    ).subscribe(() => {
      this.archivingTask = false;
      this.router.navigate(['task_pools', this.task.pool.id])
      this.notificationService.showSuccess("Aufgabe erfolgreich archiviert.");
    }, error => {
      this.archivingTask = false;
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Archivieren der Aufgabe",
          )
        );
      } else {
        throw error;
      }
    })
  }
}
