package de.hsrm.sls.subato.attendances.attendance;

import de.hsrm.sls.subato.attendances.attendance.dto.*;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.attendance.AttendancePermission;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.CourseService;
import de.hsrm.sls.subato.courses.event.CourseUnitEventNotFoundException;
import de.hsrm.sls.subato.courses.event.CourseUnitEventService;
import de.hsrm.sls.subato.courses.unit.CourseUnitNotFoundException;
import de.hsrm.sls.subato.courses.unit.CourseUnitService;
import de.hsrm.sls.subato.user.UserNotFoundException;
import de.hsrm.sls.subato.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;

import static org.springframework.http.HttpStatus.BAD_REQUEST;

@RestController
@Tag(name = "Anwesenheitserfassung")
public class AttendanceController {
    @Autowired
    private ParticipantAttendanceMapper participantAttendanceMapper;

    @Autowired
    private AttendanceMapper attendanceMapper;

    @Autowired
    private AttendanceService attendanceService;

    @Autowired
    private CourseService courseService;

    @Autowired
    private UserService userService;

    @Autowired
    private CourseUnitEventService courseUnitEventService;

    @Autowired
    private CourseUnitService courseUnitService;

    @Autowired
    private PolicyEvaluator policy;

    @GetMapping("/attendance/course")
    @Operation(summary = "Zusammenfassung der Anwesenheitsinformationen des Benutzers in allen Kurseinheiten (z.B. Praktikumsgruppen)")
    List<ParticipantAttendanceDetailDto> getCourseAttendance(@RequestParam Long courseId, @RequestParam Long userId) {
        try {
            var course = courseService.findById(courseId);
            var user = userService.findById(userId);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(Pair.of(course, user), AttendancePermission.ATTENDANCE_VIEW));

            var courseUnitAttendance = attendanceService.getAttendanceForCourseAndUser(course, user);
            return courseUnitAttendance.stream()
                   .map(participantAttendance -> participantAttendanceMapper.map(participantAttendance))
                   .toList();
        } catch(CourseNotFoundException ex) {
           throw new ResponseStatusException(BAD_REQUEST, "Kurs existiert nicht");
        }
    }

    @GetMapping("/attendance/unit")
    @Operation(summary = "Zusammenfassung der Anwesenheit des Benutzers innerhalb einer Kurseinheit (z.B. Praktikumsgruppe)")
    ParticipantAttendanceDetailDto getCourseUnitAttendance(@RequestParam Long unitId, @RequestParam Long userId) {
        try {
            var courseUnit = courseUnitService.findById(unitId);
            var user = userService.findById(userId);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(Pair.of(courseUnit.getCourse(), user), AttendancePermission.ATTENDANCE_VIEW));

            var courseUnitAttendance = attendanceService.getAttendanceForUnitAndUser(courseUnit, user);
            return participantAttendanceMapper.map(courseUnitAttendance);
        } catch(CourseUnitNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Kurseinheit existiert nicht");
        }
    }

    @PutMapping("/attendance")
    AttendanceDto createAttendance(@RequestBody CreateAttendanceDto dto) {
        try {
            var event = courseUnitEventService.findById(dto.getEventId());
            var user = userService.findById(dto.getUserId());

            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(event, AttendancePermission.ATTENDANCE_UPDATE));

            var attendance = attendanceService.createAttendance(user, event);
            return attendanceMapper.map(attendance);
        } catch(CourseUnitEventNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Termin existiert nicht");
        } catch(UserNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Benutzer existiert nicht");
        }
    }

    @DeleteMapping("/attendance/user/{userId}/event/{eventId}")
    void deleteAttendance(@PathVariable Long userId, @PathVariable Long eventId) {
        try {
            var event = courseUnitEventService.findById(eventId);
            var user = userService.findById(userId);

            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(event, AttendancePermission.ATTENDANCE_UPDATE));

            attendanceService.deleteAttendance(user, event);
        } catch(AttendanceNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, AttendanceNotFoundException.REASON);
        }
    }
}
