package de.hsrm.sls.subato.courses.course;

import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.courses.membership.*;
import de.hsrm.sls.subato.courses.unit.*;
import de.hsrm.sls.subato.modules.progmodule.ProgramModuleRepository;
import de.hsrm.sls.subato.modules.term.TermService;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserFilter;
import de.hsrm.sls.subato.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import java.util.List;


@Service
public class CourseService {
    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private TermService termService;

    @Autowired
    private UserService userService;

    @Autowired
    private ProgramModuleRepository programModuleRepository;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private DefaultDescriptionRepository defaultDescriptionRepository;

    @Autowired
    private CourseUnitService courseUnitService;

    @Autowired
    private CourseStaffMembershipService staffMembershipService;

    @Autowired
    private CourseCopyService courseCopyService;

    public Course findById(long id) {
        return findById(id, true);
    }

    public Course findById(long id, boolean raiseException) {
        var course = courseRepository.findById(id);

        if(raiseException) {
            return course.orElseThrow(() -> new CourseNotFoundException(id));
        } else {
            return course.orElse(null);
        }
    }

    public List<Course> findAll(CourseFilter filter, String permission) {
        Specification<Course> spec = filter.toSpecification();

        if(permission != null) {
            var authSpec = policy.checkAsSpec(CheckPolicyRequest.fromPrincipal(null, permission));
            spec = spec.and(authSpec);
        }

        return courseRepository.findAll(spec);
    }

    public boolean courseExists(long id) {
        return courseRepository.existsById(id);
    }

    public List<Course> findParticipatingCourses(User user, Long termId) {
        var participatingCourseIds = user.getParticipatingUnits().stream()
                .map(unit -> unit.getCourse())
                .filter(c -> c.getTerm() == null || (termId != null && c.getTerm().getId() == termId))
                .map(c -> c.getId())
                .distinct()
                .toList();

        return courseRepository.findAllById(participatingCourseIds);
    }

    public List<Course> findCoursesWhereIsMember(User user, Long termId) {
        var courseIds = user.getStaffMemberships().stream()
                .map(m -> m.getCourse())
                .distinct()
                .filter(c -> c.getTerm() == null || (termId != null && c.getTerm().getId() == termId))
                .map(c -> c.getId())
                .toList();

        return courseRepository.findAllById(courseIds);
    }


    /**
     * @throws de.hsrm.sls.subato.modules.term.TermNotFoundException
     * @throws de.hsrm.sls.subato.user.UserNotFoundException
     * @throws CourseNotFoundException if the template course was not found
     */
    public Course create(CreateCourseRequest dto, SubatoPrincipal principal) {
        var templateCourse = dto.getCourseId() != null ? findById(dto.getCourseId()) : null;
        var user = principal != null ? userService.findByPrincipal(principal) : null;
        var term = dto.getTermId() != null ? termService.findById(dto.getTermId()) : null;
        var submitPolicy = dto.getSubmitPolicy() == null ?
                dto.getTermId() == null ? SubmitPolicy.EVERYONE : SubmitPolicy.PARTICIPANTS
                : dto.getSubmitPolicy();
        var visibility = dto.getVisibility() == null ? CourseVisibility.PRIVATE : dto.getVisibility();

        var course = new Course();
        course.setName(dto.getName());
        course.setShortName(dto.getShortName());
        course.setShowEmails(dto.isShowEmails());
        course.setDescription(defaultDescriptionRepository.getDefaultDescription());
        course.setVisibility(visibility);
        course.setSubmitPolicy(submitPolicy);
        course.setTerm(term);
        course.setSourceId(dto.getSourceId());

        var modules = programModuleRepository.findAllById(dto.getModuleIds());
        course.setModules(modules);

        course = courseRepository.save(course);
        createDefaultUnits(course);

        if(user != null) {
            staffMembershipService.addMembership(course, user, StaffGroup.ADMIN);
        }

        if(templateCourse != null) {
            courseCopyService.copyCourse(templateCourse, course);
        }

        var supplement = CourseSupplementFactory.create();
        if(supplement != null) {
            course = supplement.supplement(course);
        }

        return course;
    }

    public Course update(Course course, UpdateCourseDto dto) {
        course.setDescription(dto.getDescription());
        course.setName(dto.getName());
        course.setShortName(dto.getShortName());
        course.setMotd(dto.getMotd());
        course.setVisibility(dto.getVisibility());
        course.setShowEmails(dto.isShowEmails());
        course.setSubmitPolicy(dto.getSubmitPolicy());

        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(course);
        if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
        }

        return courseRepository.save(course);
    }

    public Course reimport(Course course) {
        var supplement = CourseSupplementFactory.create();
        if(supplement != null) {
            course = supplement.supplement(course);
        }

        return course;
    }

    /**
     * @throws CourseNotFoundException
     */
    public void delete(Course course) {
        courseRepository.delete(course);
    }

    private void createDefaultUnits(Course course) {
        if(course.getSandbox() == null) {
            var request = new CreateCourseUnitRequestBuilder()
                    .setName("Sandkasten")
                    .setCourse(course)
                    .setType(CourseUnitType.Misc)
                    .build();
            courseUnitService.create(request);
        }

        if(course.getDefault() == null) {
            var request = new CreateCourseUnitRequestBuilder()
                    .setName("Standard")
                    .setCourse(course)
                    .setType(CourseUnitType.Misc)
                    .build();
            courseUnitService.create(request);
        }
    }

    List<User> getParticipants(Course course) {
        return userService.findAll(new UserFilter().courseId(course.getId()), userService.nameSort());
    }
}
