package de.hsrm.sls.subato.shared.error;

import de.hsrm.sls.subato.shared.DangerousActionException;
import de.hsrm.sls.subato.shared.ExceptionReasons;
import org.apache.catalina.connector.ClientAbortException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.multipart.MaxUploadSizeExceededException;
import org.springframework.web.server.ResponseStatusException;

import javax.servlet.MultipartConfigElement;
import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolationException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

@ControllerAdvice
public class GlobalExceptionHandler {
    private static Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    final String NOT_AUTHENTICATED_MESSAGE = "Nicht eingeloggt.";
    final String UNAUTHORIZED_MESSAGE = "Keine Rechte für diese Aktion.";

    @Autowired
    MultipartConfigElement multipartConfigElement;

    @ExceptionHandler(MaxUploadSizeExceededException.class)
    public ResponseEntity<ErrorResponse> maxUploadSizeExceededException(MaxUploadSizeExceededException ex) {
        var statusCode = HttpStatus.BAD_REQUEST;

        var maxFileSizeInMb = multipartConfigElement.getMaxFileSize() / 1024 / 1024;
        var errorResponse = new ErrorResponse(statusCode.value(), ExceptionReasons.MAX_UPLOAD_SIZE_EXCEEDED.formatted(maxFileSizeInMb), null);
        return ResponseEntity.status(statusCode.value()).body(errorResponse);
    }

    // https://stackoverflow.com/questions/35451244/how-to-handle-clientabortexception-in-spring-mvc
    @ExceptionHandler(ClientAbortException.class)
    public void handleLockException(ClientAbortException exception, HttpServletRequest request) {
        final String message = "ClientAbortException generated by request {} {} from remote address {} with X-FORWARDED-FOR {}";
        final String headerXFF = request.getHeader("X-FORWARDED-FOR");
        logger.debug(message, request.getMethod(), request.getRequestURL(), request.getRemoteAddr(), headerXFF);
    }

    @ExceptionHandler(ConstraintViolationException.class)
    public ResponseEntity<ErrorResponse> constraintViolationException(ConstraintViolationException ex) throws IOException {
        var statusCode = HttpStatus.BAD_REQUEST;
        Map<String, String> errors = new HashMap<>();
        ex.getConstraintViolations().forEach((error) -> {
            String fieldName = error.getPropertyPath().toString();
            String errorMessage = error.getMessage();
            errors.put(fieldName, errorMessage);
        });
        var errorResponse = new ErrorResponse(statusCode.value(), ExceptionReasons.CONSTRAINT_VALIDATION, ErrorCode.VALIDATION_FAILURE, errors);
        return ResponseEntity.status(statusCode.value()).body(errorResponse);
    }

    @ExceptionHandler(DangerousActionException.class)
    public ResponseEntity<ErrorResponse> dangerousActionException(DangerousActionException ex) throws IOException {
        var statusCode = HttpStatus.BAD_REQUEST;
        Map<String, String> errors = new HashMap<>();
        errors.put("description", ex.getDescription());
        errors.put("suggestion", ex.getSuggestion());
        var errorResponse = new ErrorResponse(statusCode.value(), DangerousActionException.REASON, ErrorCode.DANGEROUS_ACTION, errors);
        return ResponseEntity.status(statusCode.value()).body(errorResponse);
    }

    @ExceptionHandler(AccessDeniedException.class)
    private ResponseEntity<ErrorResponse> handleAccessDenied(AccessDeniedException e) {
        logger.debug("Resolved exception: %s".formatted(e.getMessage()), e);
        var isAuthenticated = !(SecurityContextHolder.getContext().getAuthentication() instanceof AnonymousAuthenticationToken);
        var statusCode = isAuthenticated ? HttpStatus.FORBIDDEN : HttpStatus.UNAUTHORIZED;
        var message = isAuthenticated ? UNAUTHORIZED_MESSAGE : NOT_AUTHENTICATED_MESSAGE;

        var errorResponse = new ErrorResponse(statusCode.value(), message, null);
        return ResponseEntity.status(statusCode.value()).body(errorResponse);
    }

    @ExceptionHandler(ResponseStatusException.class)
    private ResponseEntity<ErrorResponse> handleResponseStatus(ResponseStatusException e) {
        if(e.getStatus() == HttpStatus.INTERNAL_SERVER_ERROR) {
            logger.error(e.getMessage(), e.getCause());
        } else {
            logger.debug("Resolved exception: %s".formatted(e.getMessage()), e.getCause());
        }

        return ResponseEntity.status(e.getStatus()).body(new ErrorResponse(e.getRawStatusCode(), e.getReason(), null));
    }

    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    @ExceptionHandler(Exception.class)
    public ResponseEntity<ErrorResponse> handle(Exception e) {
        // If the exception is annotated with @ResponseStatus rethrow it and let
        // the framework handle it - like the OrderNotFoundException example
        // at the start of this post.
        // AnnotationUtils is a Spring Framework utility class.
        var annotation = AnnotationUtils.findAnnotation(e.getClass(), ResponseStatus.class);
        if (annotation != null) {
            String reason = annotation.reason() != null && !annotation.reason().isEmpty() ? annotation.reason() : e.getMessage();
            var errorResponse = new ErrorResponse(annotation.value().value(), reason, null, e);
            logger.debug("Resolved exception: %s".formatted(e.getMessage()), e);

            return ResponseEntity.status(annotation.value()).body(errorResponse);
        }

        logger.error("Nicht abgefangene Exception", e);
        var errorResponse = new ErrorResponse(500, "Unbekannter Fehler", null);
        return ResponseEntity.status(500).body(errorResponse);
    }
}
