package de.hsrm.sls.subato.solutions.submit;

import com.google.common.base.Strings;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.unit.CourseUnitService;
import de.hsrm.sls.subato.exercises.submission.SubmissionService;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceService;
import de.hsrm.sls.subato.solutions.comment.SolutionComment;
import de.hsrm.sls.subato.solutions.evaluation.EvaluationFailedException;
import de.hsrm.sls.subato.solutions.evaluation.InvalidSolutionException;
import de.hsrm.sls.subato.solutions.evaluation.SolutionEvaluationService;
import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.solutions.solution.SolutionContentRepository;
import de.hsrm.sls.subato.solutions.solution.SolutionRepository;
import de.hsrm.sls.subato.user.UserService;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import java.time.LocalDateTime;

/**
 * handles the submission of a solution to an exercise by storing it in the database,
 * eventually adding the solution to the submission from the submitting user for the affected exercise,
 * adding the submitting user as a participant to the course and
 * evaluating the solution if an evaluator has been configured for the task
 *
 * The solution will only be added to the submission if it is not delayed or the attempts are not exceeded.
 * Except if the submitting user has explicitly stated so. If a submission does not exist yet, it will be created.
 */
@Service
public class ExerciseSubmitHandler implements SubmitHandler {
    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private SolutionContentRepository solutionContentRepository;

    @Autowired
    private UserService userService;

    @Autowired
    private SubmissionService submissionService;

    @Autowired
    private SolutionEvaluationService solutionEvaluationService;

    @Autowired
    private CourseUnitService courseUnitService;

    @Autowired
    private PolicyEvaluator policy;


    /**
     * @throws EvaluationFailedException if the evaluation of the solution failed
     * @throws InvalidSolutionException if the solution cannot be evaluated because it was incorrectly submitted
     */
    @Override
    public Solution handle(SubmitSolutionRequest request, SubatoPrincipal principal, LocalDateTime submittedAt) {
        policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(Pair.of(request.exercise(), request.task()), SolutionsAction.SOLUTION_SUBMIT_EXERCISE));
        var user = principal != null ? userService.findById(principal.getId()) : null;

        Solution solution = new Solution();
        solution.setSubmittedBy(user);
        solution.setTask(request.task());
        solution.setSubmittedAt(submittedAt);
        solution.setSubmittedWith(request.client());
        solution.setExercise(request.exercise());

        if(user != null && !Strings.isNullOrEmpty(request.comment())) {
            var comment = new SolutionComment();
            comment.setText(request.comment());
            comment.setFrom(user);
            comment.setSolution(solution);
            comment.setCreatedAt(LocalDateTime.now());
            solution.getComments().add(comment);
        }

        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(solution);
        if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
        }

        solution = solutionRepository.save(solution);

        if(user != null) {
            addAsParticipantIfNoParticipant(principal, request.exercise().getCourse());

            var currentAttempts = submissionService.getCurrentAttempts(request.task(), request.exercise(), user);
            var attemptsExceeded = request.task().getAttempts() != null && currentAttempts > request.task().getAttempts();
            solution.setAttemptsExceeded(attemptsExceeded);

            solution.setDelayed(request.exercise().getDueTo() != null && submittedAt.isAfter(request.exercise().getDueTo()));
            // we eventually add the solution to the submission
            if (!solution.isDelayed() && !attemptsExceeded || request.lateSubmit()) {
                submissionService.processSolution(solution);
            }

            solution = solutionRepository.save(solution);
        }

        solutionContentRepository.store(request.content(), solution);

        // eventually evaluate the solution if the task is configured for automated evaluation
        var task = solution.getTask();
        if(task.evaluationPossible()) {
            var evalResult = solutionEvaluationService.test(solution);
            solution.setEvalResult(evalResult);
            return solutionRepository.save(solution);
        }

        return solution;
    }

    private void addAsParticipantIfNoParticipant(SubatoPrincipal principal, Course course) {
        if(!principal.isParticipant(course.getId())) {
            var defaultUnit = course.getDefault();
            var user = userService.findById(principal.getId());
            courseUnitService.addParticipant(defaultUnit, user);
        }
    }
}
