package de.hsrm.sls.subato.tasks.pool;

import de.hsrm.sls.subato.courses.course.CourseDetailDto;
import de.hsrm.sls.subato.courses.course.CourseMapper;
import de.hsrm.sls.subato.shared.ExceptionReasons;
import de.hsrm.sls.subato.tasks.imp.TaskImportException;
import de.hsrm.sls.subato.tasks.imp.TaskImportResult;
import de.hsrm.sls.subato.tasks.imp.TaskImportService;
import de.hsrm.sls.subato.tasks.pool.sync.repository.RepositoryDoesNotExistException;
import de.hsrm.sls.subato.tasks.pool.sync.TaskPoolSyncException;
import de.hsrm.sls.subato.tasks.pool.sync.TaskPoolSyncService;
import de.hsrm.sls.subato.tasks.imp.log.TaskImportLogDto;
import de.hsrm.sls.subato.tasks.imp.log.TaskImportLogMapper;
import de.hsrm.sls.subato.tasks.pool.sync.repository.RepositoryException;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.task.TaskDto;
import de.hsrm.sls.subato.tasks.task.TaskMapper;
import de.hsrm.sls.subato.tasks.task.TaskNotFoundException;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static org.springframework.http.HttpStatus.*;

@RestController
@Tag(name = "Aufgabenverwaltung")
public class TaskPoolController {
    @Autowired
    private TaskPoolMapper taskPoolMapper;

    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private CourseMapper courseMapper;

    @Autowired
    private TaskImportService taskImportService;

    @Autowired
    private TaskPoolService taskPoolService;

    @Autowired
    private TaskPoolSyncService taskPoolSyncService;

    @Autowired
    private TaskImportLogMapper taskImportLogMapper;

    @GetMapping("/task_pool/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskPoolNotFoundException.REASON),
    })
    TaskPoolDto getOne(@PathVariable Long id) {
        try {
            var pool = taskPoolService.findById(id);
            return taskPoolMapper.map(pool);
        } catch(TaskPoolNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskPoolNotFoundException.REASON);
        }
    }

    @GetMapping("/task_pool/{id}/tasks")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    @Operation(summary = "Liefert eine Übersicht aller unarchivierten Aufgaben aus dem Aufgaben-Pool")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskPoolNotFoundException.REASON),
    })
    Page<TaskDto> getTasks(@PageableDefault Pageable page, @PathVariable Long id, @RequestParam("q") Optional<String> query) {
        try {
            var tasks = taskPoolService.getTasks(id, page, query);
            return tasks.map(c -> taskMapper.map(c));
        } catch(TaskPoolNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskPoolNotFoundException.REASON);
        }
    }

    @GetMapping("/task_pool/{id}/courses")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    @Operation(summary = "Liefert eine Übersicht aller Kurse, die mindestens eine Aufgabe aus dem angegebenen Aufgaben-Pool in einem Übungsblatt verwendet haben")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskPoolNotFoundException.REASON),
    })
    List<CourseDetailDto> getCourses(@PathVariable Long id) {
        try {
            var courses = taskPoolService.getCourses(id);
            return courses.stream().map(c -> courseMapper.mapDetail(c)).toList();
        } catch(TaskPoolNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskPoolNotFoundException.REASON);
        }
    }

    @GetMapping("/task_pool/{id}/logs")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    @Operation(summary = "Liefert Logs, die beim Import entstanden sind.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskPoolNotFoundException.REASON),
    })
    List<TaskImportLogDto> getLogs(@PathVariable Long id) {
        try {
            var logs = taskPoolService.getLogs(id);
            return logs.stream().map(c -> taskImportLogMapper.map(c)).toList();
        } catch(TaskPoolNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskPoolNotFoundException.REASON);
        }
    }

    @GetMapping("/task_pool")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    List<TaskPoolDto> getAll() {
        var pools = taskPoolService.getAll();
        return pools.stream().map(task -> taskPoolMapper.map(task)).collect(Collectors.toList());
    }

    @PutMapping("/task_pool/{id}/tasks")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_UPDATE)")
    @Operation(
            summary = "Importiert eine Aufgabe aus der übermittelten STEF-Spezifikation (als zip) in den Aufgaben-Pool bzw. aktualisiert sie",
            description = "Ob eine Aufgabe bereits existiert, wird anhand des id-Attributs in meta.xml festgestellt"
    )
    @ApiResponses(value = {
            @ApiResponse(responseCode = "400", description = InvalidSpecException.REASON + " oder " + TaskPoolReadyOnlyException.REASON),
            @ApiResponse(responseCode = "404", description = TaskPoolNotFoundException.REASON),
            @ApiResponse(responseCode = "500", description = TaskImportException.REASON),
    })
    TaskDto importTask(@PathVariable Long id, @RequestPart("file") MultipartFile file) throws TaskImportException {
        try {
            var pool = taskPoolService.findById(id);
            if(pool.isLinkedToRepository()) {
                throw new TaskPoolReadyOnlyException();
            }


            var task = taskImportService.importTask(pool, file);
            return taskMapper.map(task);
        } catch(TaskPoolNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskPoolNotFoundException.REASON);
        } catch(InvalidSpecException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "%s: %s".formatted(InvalidSpecException.REASON, ex.getMessage()));
        } catch(TaskImportException ex) {
            throw new ResponseStatusException(INTERNAL_SERVER_ERROR, TaskImportException.REASON);
        } catch(TaskPoolReadyOnlyException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskPoolReadyOnlyException.REASON);
        }
    }

    @PostMapping("/task_pool")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_CREATE)")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "400", description = ExceptionReasons.CONSTRAINT_VALIDATION + " oder " + RepositoryDoesNotExistException.REASON),
            @ApiResponse(responseCode = "500", description = RepositoryException.REASON),
    })
    CreateTaskPoolResultDto create(@RequestBody CreateTaskPoolDto dto) {
        TaskPool pool;

        try {
            pool = taskPoolService.create(dto);
        } catch (RepositoryDoesNotExistException e) {
            throw new ResponseStatusException(BAD_REQUEST, RepositoryDoesNotExistException.REASON);
        } catch (RepositoryException e) {
            throw new ResponseStatusException(INTERNAL_SERVER_ERROR, RepositoryException.REASON);
        }

        if(pool.isLinkedToRepository()) {
            try {
                var result = taskPoolSyncService.syncPool(pool, false);
                return new CreateTaskPoolResultDto(taskPoolMapper.map(pool), false, result);
            } catch (TaskPoolSyncException e) {
                return new CreateTaskPoolResultDto(taskPoolMapper.map(pool), true);
            }
        }

        return new CreateTaskPoolResultDto(taskPoolMapper.map(pool), false);
    }

    @PutMapping ("/task_pool/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_UPDATE)")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "400", description = ExceptionReasons.CONSTRAINT_VALIDATION + " oder " + RepositoryDoesNotExistException.REASON),
            @ApiResponse(responseCode = "404", description = TaskPoolNotFoundException.REASON),
            @ApiResponse(responseCode = "500", description = RepositoryException.REASON)
    })
    TaskPoolDto update(@PathVariable Long id, @RequestBody CreateTaskPoolDto dto) {
        try {
            var taskPool = taskPoolService.update(id, dto);
            return taskPoolMapper.map(taskPool);
        } catch(TaskPoolNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskPoolNotFoundException.REASON);
        } catch (RepositoryDoesNotExistException e) {
            throw new ResponseStatusException(BAD_REQUEST, RepositoryDoesNotExistException.REASON);
        } catch (RepositoryException e) {
            throw new ResponseStatusException(INTERNAL_SERVER_ERROR, RepositoryException.REASON);
        }
    }

    @DeleteMapping("/task_pool/{id}")
    @Operation(summary = "Löscht einen Aufgaben-Pool mit allen Aufgaben, verknüpfte Lösungen etc.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskPoolNotFoundException.REASON),
    })
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_DELETE)")
    void delete(@PathVariable Long id) {
        try {
            taskPoolService.delete(id);
        } catch(TaskPoolNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskNotFoundException.REASON);
        }
    }

    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_UPDATE)")
    @PutMapping("/task_pool/{id}/sync")
    TaskImportResult syncPool(@PathVariable Long id) {
        try{
            var pool = taskPoolService.findById(id);
            return taskPoolSyncService.syncPool(pool, true);
        } catch(TaskPoolNotFoundException e){
            throw new ResponseStatusException(NOT_FOUND, TaskPoolNotFoundException.REASON);
        } catch (RepositoryDoesNotExistException e) {
            throw new ResponseStatusException(BAD_REQUEST, RepositoryDoesNotExistException.REASON);
        } catch (TaskPoolSyncException e) {
            throw new ResponseStatusException(INTERNAL_SERVER_ERROR, RepositoryDoesNotExistException.REASON);
        }
    }
}
