package de.hsrm.sls.subato.tasks.stef.parser.meta;

import com.google.common.base.Strings;
import de.hsrm.sls.subato.tasks.stef.Meta;
import de.hsrm.sls.subato.tasks.stef.SubmissionMode;
import de.hsrm.sls.subato.tasks.stef.SubmissionModes;
import de.hsrm.sls.subato.tasks.stef.file.FileType;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.SubmissionModeXml;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.SubmissionModesXml;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.TaskXml;
import de.hsrm.sls.subato.tasks.task.SubmissionModeType;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class SubmissionModeExtractor {
    public void extract(TaskXml doc, Meta meta) {
        if (doc.getSubmissionModes() != null) {
            if (doc.getSubmissionModes().getSubmissionModes().size() == 0) {
                throw new InvalidSpecException("Wenn %s konfiguriert wird, muss mind. ein %s konfiguriert werden".formatted(TaskXml.SUBMISSION_MODES, SubmissionModesXml.SUBMISSION_MODE));
            }

            var submissionModes = new ArrayList<SubmissionMode>();
            for (var xmlSubmissionMode : doc.getSubmissionModes().getSubmissionModes()) {
                if (xmlSubmissionMode.getName() == null) {
                    throw new InvalidSpecException("%s-Attribut in %s fehlt".formatted(SubmissionModeXml.NAME, SubmissionModesXml.SUBMISSION_MODE));
                }

                try {
                    var type = SubmissionModeType.valueOf(xmlSubmissionMode.getName());
                    var submissionMode = new SubmissionMode();
                    submissionMode.setType(type);

                    var extensionsString = xmlSubmissionMode.getExtensions();
                    if(type == SubmissionModeType.TEXT && !Strings.isNullOrEmpty(extensionsString)) {
                        throw new InvalidSpecException("Angabe von %s für %s %s nicht möglich".formatted(SubmissionModeXml.EXTENSIONS, SubmissionModesXml.SUBMISSION_MODE, SubmissionModeType.TEXT));
                    }

                    if (type == SubmissionModeType.FILE) {
                        if (!Strings.isNullOrEmpty(extensionsString)) {
                            var extensions = Arrays.stream(extensionsString.split(";")).toList();
                            submissionMode.setExtensions(extensions);
                        } else {
                            submissionMode.setExtensions(getDefaultExtensions(meta));
                        }
                    }

                    submissionModes.add(submissionMode);
                } catch (IllegalArgumentException ex) {
                    throw new InvalidSpecException("SubmissionMode %s existiert nicht im System".formatted(xmlSubmissionMode.getName()));
                }
            }

            SubmissionModeType defaultMode = null;
            if (doc.getSubmissionModes().getDefaultMode() != null) {
                try {
                    defaultMode = SubmissionModeType.valueOf(doc.getSubmissionModes().getDefaultMode());

                    var isDefaultModeConfigured = submissionModes.stream().map(m -> m.getType()).toList().contains(defaultMode);
                    if (!isDefaultModeConfigured) {
                        throw new InvalidSpecException("SubmissionMode %s kann nicht als default verwendet werden, da er nicht konfiguriert wurde".formatted(doc.getSubmissionModes().getDefaultMode()));
                    }
                } catch (IllegalArgumentException ex) {
                    throw new InvalidSpecException("SubmissionMode %s existiert nicht im System".formatted(doc.getSubmissionModes().getDefaultMode()));
                }
            }

            meta.setSubmissionModes(new SubmissionModes(submissionModes, defaultMode != null ? defaultMode : getDefaultMode(submissionModes)));
        } else {
            var modes = new ArrayList<SubmissionMode>();

            var fileSubmissionMode = new SubmissionMode(SubmissionModeType.FILE);
            fileSubmissionMode.setExtensions(getDefaultExtensions(meta));
            modes.add(fileSubmissionMode);

            if (meta.getFiles().stream().filter(f -> f.getType() == FileType.TEMPLATE).count() <= 1) {
                modes.add(new SubmissionMode(SubmissionModeType.TEXT));
            }

            var submissionModes = new SubmissionModes(modes, getDefaultMode(modes));
            meta.setSubmissionModes(submissionModes);
        }
    }

    private SubmissionModeType getDefaultMode(List<SubmissionMode> modes) {
        return modes.stream().map(m -> m.getType()).toList().contains(SubmissionModeType.TEXT)
                ? SubmissionModeType.TEXT
                : SubmissionModeType.FILE;
    }

    private List<String> getDefaultExtensions(Meta meta) {
        var extensions = new ArrayList<String>();
        extensions.add("zip");

        var templateExtensions = meta.getFiles().stream()
                .filter(f -> f.getType() == FileType.TEMPLATE)
                .map(f -> f.getFile().getExtension())
                .collect(Collectors.toList());

        extensions.addAll(templateExtensions);
        return extensions;
    }
}
